from typing import Any, Dict, List, Optional

from pydantic import BaseModel

from ..collection import Collection
from ..item import ItemCollection


class SearchObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v0.9.0/extensions/single-file-stac#search-object
    """

    endpoint: Optional[str]
    parameters: Optional[Dict[Any, Any]]


class SingleFileStac(ItemCollection):
    """
    https://github.com/radiantearth/stac-spec/tree/v0.9.0/extensions/single-file-stac#single-file-stac-specification
    """

    collections: List[Collection]
    search: SearchObject

    def to_dict(self):
        return self.dict(by_alias=True, exclude_unset=True)



from typing import Any, Dict

from pydantic import BaseModel


class CommonsExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v0.9.0/extensions/commons
    """

    properties: Dict[Any, Any]



from .assets import AssetExtension
from .commons import CommonsExtension
from .datacube import DatacubeExtension
from .eo import ElectroOpticalExtension
from .label import LabelExtension
from .pc import PointCloudExtension
from .proj import ProjectionExtension
from .sar import SARExtension
from .sat import SatelliteExtension
from .sci import ScientificExtension
from .version import VersionExtension
from .view import ViewExtension


class Extensions:
    asset = AssetExtension
    commons = CommonsExtension
    datacube = DatacubeExtension
    eo = ElectroOpticalExtension
    label = LabelExtension
    pointcloud = PointCloudExtension
    proj = ProjectionExtension
    sar = SARExtension
    sat = SatelliteExtension
    scientific = ScientificExtension
    version = VersionExtension
    view = ViewExtension

    aliases = {}

    @classmethod
    def register(cls, k, v, alias=None):
        setattr(cls, k, v)

        if alias:
            cls.aliases[alias] = k

    @classmethod
    def get(cls, k):
        try:
            return getattr(cls, k)
        except AttributeError:
            try:
                return getattr(cls, cls.aliases[k])
            except KeyError:
                raise AttributeError(f"Invalid extension name or alias: {k}")



from typing import Dict, List, Optional

from pydantic import BaseModel

from ..shared import AssetRoles


class CollectionAsset(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v0.0.9/extensions/asset#asset-object
    """

    type: Optional[str]
    title: Optional[str]
    description: Optional[str]
    roles: Optional[List[AssetRoles]]


class AssetExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v0.0.9/extensions/asset#asset-definition-specification
    """

    assets: Dict[str, CollectionAsset]



from enum import Enum, auto
from typing import List, Optional, Tuple, Union

from pydantic import BaseModel, Extra, Field

from .utils import AutoValueEnum

NumType = Union[float, int]
BBox = Union[
    Tuple[NumType, NumType, NumType, NumType],  # 2D bbox
    Tuple[NumType, NumType, NumType, NumType, NumType, NumType],  # 3D bbox
]

# https://tools.ietf.org/html/rfc3339#section-5.6
DATETIME_RFC339 = "%Y-%m-%dT%H:%M:%SZ"


class ExtensionTypes(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/blob/v0.9.0/extensions/README.md#list-of-content-extensions
    """

    asset = auto()
    checksum = auto()
    commons = auto()
    context = auto()
    cube = auto()
    eo = auto()
    label = auto()
    pc = auto()
    proj = auto()
    sar = auto()
    sat = auto()
    sci = auto()
    version = auto()
    view = auto()


class MimeTypes(str, Enum):
    """
    https://github.com/radiantearth/stac-spec/blob/v0.9.0/item-spec/item-spec.md#media-types
    """

    # Raster
    geotiff = "image/tiff; application=geotiff"
    cog = "image/geo+tiff; application=geotiff; profile=cloud-optimized"
    jp2 = "image/jp2"
    png = "image/png"
    jpeg = "image/jpeg"
    # Vector
    geojson = "application/geo+json"
    geopackage = "application/geopackage+sqlite3"
    kml = "application/vnd.google-earth.kml+xml"
    kmz = "application/vnd.google-earth.kmz"
    # Others
    hdf = "application/x-hdf"
    hdf5 = "application/x-hdf5"
    xml = "application/xml"
    json = "application/json"
    html = "text/html"
    text = "text/plain"


class Relations(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/blob/v0.9.0/collection-spec/collection-spec.md#relation-types
    """

    self = auto()
    root = auto()
    parent = auto()
    child = auto()
    item = auto()
    license = auto()
    derived_from = auto()
    collection = auto()
    alternate = auto()
    previous = auto()
    next = auto()
    conformance = auto()
    docs = auto()
    tiles = auto()
    search = auto()


class AssetRoles(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/blob/v0.9.0/extensions/asset/README.md
    """

    thumbnail = auto()
    overview = auto()
    data = auto()
    metadata = auto()


class Link(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v0.9.0/collection-spec/collection-spec.md#link-object
    """

    href: str
    rel: str
    type: Optional[str]
    title: Optional[str]
    # Label extension
    label: Optional[str] = Field(None, alias="label:assets")

    class Config:
        use_enum_values = True


class Asset(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v0.9.0/item-spec/item-spec.md#asset-object
    """

    href: str
    type: Optional[str]
    title: Optional[str]
    description: Optional[str]
    roles: Optional[List[AssetRoles]]
    # EO extension
    bands: Optional[List[int]] = Field(None, alias="eo:bands")
    # SAR extension
    polarizations: Optional[List[str]] = Field(None, alias="sar:polarizations")
    # Checksum extension
    multihash: Optional[str] = Field(None, alias="checksum:multihash")

    class Config:
        allow_population_by_field_name = True
        use_enum_values = True
        extra = Extra.allow
